package it.inaf.oats.ia2.tap.tsm.rest;

import it.inaf.oats.ia2.tap.tsm.model.Column;
import it.inaf.oats.ia2.tap.tsm.model.DbEntity;
import it.inaf.oats.ia2.tap.tsm.model.IDbEntitiesContainer;
import it.inaf.oats.ia2.tap.tsm.model.Schema;
import it.inaf.oats.ia2.tap.tsm.model.Status;
import it.inaf.oats.ia2.tap.tsm.model.Table;
import it.inaf.oats.ia2.tap.tsm.model.TapSchema;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class PageModel {

    class Entity {

        boolean toRemove;
        Map<String, EditableField> values;

        Entity(DbEntity dbEntity) {
            toRemove = dbEntity.getStatus() == Status.TO_REMOVE;
            values = getEditablesFields(dbEntity);
        }
    }

    class EditableField {

        String value;
        boolean isChanged;

        EditableField(String key, DbEntity container) {
            this.value = container.getValue(key);
            this.isChanged = container.isChanged(key);
        }
    }

    class EntityItem {

        private final String title;
        private final boolean toRemove;
        private final boolean active;

        public EntityItem(DbEntity dbEntity, boolean active) {
            this.title = dbEntity.getName();
            this.toRemove = dbEntity.getStatus() == Status.TO_REMOVE;
            this.active = active;
        }
    }

    class TapSchemaModel {

        boolean hasAddables;
        boolean hasChildren;
    }

    class SchemaModel extends Entity {

        boolean hasAddables;
        boolean hasChildren;

        public SchemaModel(Schema schema) {
            super(schema);
            if (schema != null) {
                hasAddables = !schema.getAddables().isEmpty();
                hasChildren = !schema.getAllEntities().isEmpty();
            }
        }
    }

    class TableModel extends Entity {

        boolean hasAddables;
        boolean hasChildren;

        public TableModel(Table table) {
            super(table);
            if (table != null) {
                hasAddables = !table.getAddables().isEmpty();
                hasChildren = !table.getAllEntities().isEmpty();
            }
        }
    }

    class ColumnModel extends Entity {

        boolean isPrimaryKey;
        boolean isForeignKey;
        String dataType;
        int size;

        ColumnModel(Column column) {
            super(column);

            isPrimaryKey = column.isPrimaryKey();
            isForeignKey = column.isForeignKey();
            dataType = column.getDatatype();
            size = column.getSize();
        }
    }

    TapSchemaModel tapSchema;

    SchemaModel schema;
    TableModel table;
    ColumnModel column;

    List<EntityItem> schemas;
    List<EntityItem> tables;
    List<EntityItem> columns;

    public PageModel(TapSchema tapSchema) {

        Schema selectedSchema = tapSchema.getSelectedSchema();
        Table selectedTable = tapSchema.getSelectedTable();
        Column selectedColumn = tapSchema.getSelectedColumn();

        this.tapSchema = new TapSchemaModel();
        this.tapSchema.hasAddables = !tapSchema.getAddables().isEmpty();
        this.tapSchema.hasChildren = !tapSchema.getAllEntities().isEmpty();

        schemas = getAllEntities(tapSchema);
        tables = getAllEntities(selectedSchema);
        columns = getAllEntities(selectedTable);

        schema = selectedSchema == null ? null : new SchemaModel(selectedSchema);
        table = selectedTable == null ? null : new TableModel(selectedTable);
        column = selectedColumn == null ? null : new ColumnModel(selectedColumn);
    }

    private List<EntityItem> getAllEntities(IDbEntitiesContainer container) {
        if (container == null) {
            return null;
        }

        ArrayList<EntityItem> entities = new ArrayList<EntityItem>();
        String selectedEntity = container.getSelected() == null ? null : container.getSelected().getName();
        for (DbEntity entity : container.getAllEntities()) {
            entities.add(new EntityItem(entity, entity.getName().equals(selectedEntity)));
        }
        return entities;
    }

    private Map<String, EditableField> getEditablesFields(DbEntity entity) {
        if (entity == null) {
            return null;
        }

        HashMap<String, EditableField> fields = new HashMap<String, EditableField>();
        for (String key : entity.getSupportedValues()) {
            fields.put(key, new EditableField(key, entity));
        }
        return fields;
    }
}
