(function ($, jsonpatch, angular) {

    var BASE_PATH = "webresources/tapschema/";

    var app = angular.module('editTapSchema', []);

    app.config(['$httpProvider', function ($httpProvider) {
            $httpProvider.defaults.headers.patch = {
                'Content-Type': 'application/json;charset=utf-8'
            };
        }]);

    function showAjaxError(errorResponse) {
        alert(errorResponse);
    }

    app.controller('mainController', ['$scope', '$http', function ($scope, $http) {
            $scope.model = {};
            $scope.currentAddables = {
                containerName: null,
                entities: []
            };
            $scope.ucdModel = {};

            function formPost(url, data, onSuccess) {
                $http({
                    method: 'POST',
                    url: BASE_PATH + url,
                    data: $.param(data),
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'}
                }).then(onSuccess, showAjaxError);
            }

            function updateModel(response) {
                var patches = response.data;
                if (patches.length === 1 && patches[0].path === '/') {
                    // to resolve standard version issue
                    // see https://github.com/dharmafly/jsonpatch.js/issues/1
                    patches[0].path = '';
                }
                $scope.model = jsonpatch.apply_patch($scope.model, patches);
            }

            $scope.select = function (containerName, entityName) {
                formPost('select', {
                    containerName: containerName,
                    entityName: entityName
                }, updateModel);
            };

            $scope.remove = function (containerName, entityName, event) {
                event.stopPropagation();
                formPost('remove', {
                    containerName: containerName,
                    entityName: entityName
                }, updateModel);
            };

            $scope.undoRemove = function (containerName) {
                formPost('undoremove', {containerName: containerName}, updateModel);
            };

            // add entities

            $scope.openAddablesDialog = function (containerName) {
                formPost('addables', {containerName: containerName}, function (response) {
                    $scope.currentAddables.containerName = containerName;
                    $scope.currentAddables.entities = response.data;
                    $('#addablesModal').modal('show');
                });
            };

            $scope.addEntities = function () {
                var entities = $scope.currentAddables.entities;
                var selected = [];
                for (var i = 0; i < entities.length; i++) {
                    if (entities[i].selected) {
                        selected.push(entities[i].name);
                    }
                }
                $http.post(BASE_PATH + 'add', {
                    containerName: $scope.currentAddables.containerName,
                    entities: selected
                }).then(function (response) {
                    updateModel(response);
                    $('#addablesModal').modal('hide');
                }, showAjaxError);
            };

            // Search UCD

            $scope.openUCDDialog = function () {
                formPost('searchUCD', {description: $scope.model.column.values.description}, function (response) {
                    $scope.ucdModel = response.data;
                    $('#searchUCDModal').modal('show');
                });
            };

            function searchUCD() {
                formPost('searchUCD', {description: $scope.ucdModel.description}, function (response) {
                    $scope.ucdModel = response.data;
                });
            }

            $scope.searchUCD = searchUCD;

            $scope.searchUCDByReturn = function (event) {
                if (event.keyCode === 13) {
                    searchUCD();
                }
            };
            
            $scope.explainUCD = function (ucdInfo, index) {
                formPost('explainUCD', {index: index}, function (response) {
                    ucdInfo.definition = response.data;
                });
            };

            $scope.selectUCD = function (ucd) {
                $scope.ucdModel.selectedUCD = ucd;
            };

            $scope.saveUCD = function () {
                $scope.model.column.values.ucd.value = $scope.ucdModel.selectedUCD;
                $('#searchUCDModal').modal('hide');
            };

            // Init
            $http.post(BASE_PATH + "model").then(function (response) {
                $scope.model = response.data;
            }, showAjaxError);
        }]);

    app.directive('editValue', ['$http', function ($http) {
            return {
                restrict: 'A', // search only in attributes name
                replace: true, // replace container element with this template
                template: '<input type="text" class="form-control" ng-model="editValue.value" ng-class="{changed: editValue.isChanged}"/>',
                scope: {
                    editValue: '='
                },
                link: function (scope, element, attrs) {
                    var attrSplit = attrs.editValue.split('.');
                    var entity = attrSplit[1];
                    var key = attrSplit[attrSplit.length - 1];
                    scope.$watch('editValue.value', function (value) {
                        if (typeof value !== 'undefined') { // find how to skip first watch
                            $http.post(BASE_PATH + 'edit', {
                                entity: entity,
                                key: key,
                                value: value
                            }).then(function (response) {
                                scope.editValue.isChanged = response.data;
                            }, showAjaxError);
                        }
                    });
                }
            };
        }]);

})(jQuery, jsonpatch, angular);