import it.inaf.oats.ia2.tap.tsm.datalayer.TapSchemaDL;
import it.inaf.oats.ia2.tap.tsm.model.Column;
import it.inaf.oats.ia2.tap.tsm.model.DbEntity;
import it.inaf.oats.ia2.tap.tsm.model.Schema;
import it.inaf.oats.ia2.tap.tsm.model.Status;
import it.inaf.oats.ia2.tap.tsm.model.Table;
import it.inaf.oats.ia2.tap.tsm.model.TapSchema;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.Ignore;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Ignore
public class DLTest {

    Connection connection;

    public DLTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() throws SQLException {
        connection = TestUtils.getCredentials().getConnection();
    }

    @After
    public void tearDown() throws SQLException {
        connection.close();
    }

    @Test
    public void loadAllSchemas() throws SQLException {
        ArrayList<String> allSchemas = TapSchemaDL.getAllSchemasNames(connection);
        assertTrue(allSchemas.size() > 0);

        System.out.println("All schemas:");
        printArray(allSchemas);
    }

    @Test
    public void loadAllTAPSchemas() throws SQLException {
        ArrayList<String> allTAPSchemas = TapSchemaDL.getAllTAPSchemasNames(connection, TapSchemaDL.getAllSchemasNames(connection));
        assertTrue(allTAPSchemas.size() > 0);

        System.out.println("All TAP schemas:");
        printArray(allTAPSchemas);
    }

    @Test
    public void loadNewEmptyTAPSchema() throws SQLException {
        TapSchema tapSchema = new TapSchema(connection, "TEST", new ArrayList<String>());

        boolean allSchemasNull = true;
        for (DbEntity schema : tapSchema.getAllEntities()) {
            if (schema != null) {
                allSchemasNull = false;
                break;
            }
        }

        assertTrue(tapSchema.getAllEntities().size() > 0 && allSchemasNull);
    }

    @Test
    public TapSchema loadNewTAPSchemaOneSchemaSelected() throws SQLException {

        TapSchema tapSchema = TestUtils.getNewTAPSchemaOneSchemaSelected(connection);

        int schemasNotNulls = 0;
        Schema schema = null;
        for (DbEntity s : tapSchema.getAllEntities()) {
            schemasNotNulls++;
            schema = (Schema) s;
        }

        assertEquals(schemasNotNulls, 1);
        assertNotNull(schema);
        assertTrue(schema.getAddables().size() > 0);

        // Check that only tableNames are loaded
//        boolean allTablesNull = true;
//        for (IDbEntity t : schema.getAllEntities()) {
//            if (t != null) {
//                allTablesNull = false;
//                break;
//            }
//        }
//        assertTrue(allTablesNull);
        // Test load first table
        String firstTableName = schema.getAddables().iterator().next();
        ArrayList<String> selectedTables = new ArrayList<String>();
        selectedTables.add(firstTableName);
        schema.loadEntities(connection, selectedTables);
        Table table = schema.getTable(firstTableName);
        assertNotNull(table);
        assertEquals(table.getStatus(), Status.SELECTED_TO_SAVE);

        // Check that all columns are loaded
        assertTrue(table.getAllEntities().size() > 0);
        for (DbEntity column : table.getAllEntities()) {
            assertEquals(column.getStatus(), Status.SELECTED_TO_SAVE);
        }

        return tapSchema;
    }

    @Test
    public void loadExistentTAPSchema() throws SQLException {
        String tapSchemaName = TapSchemaDL.getAllTAPSchemasNames(connection, TapSchemaDL.getAllSchemasNames(connection)).get(0);
        TapSchema tapSchema = new TapSchema(connection, tapSchemaName);
        assertNotNull(tapSchema);

        System.out.println("Loaded " + tapSchema.getAllEntities().size() + " schemas.");

        // Test loading saved schemas and tables
        for (DbEntity schema : tapSchema.getAllEntities()) {
            if (schema != null) {
                assertEquals(schema.getStatus(), Status.SELECTED_SAVED);
                System.out.println("Loaded " + ((Schema) schema).getAllEntities().size() + " tables in the first schema (" + schema.getName() + ").");
                for (DbEntity table : ((Schema) schema).getAllEntities()) {
                    if (table != null) {
                        assertEquals(table.getStatus(), Status.SELECTED_SAVED);
                        System.out.println("Loaded " + ((Table) table).getAllEntities().size() + " columns in first table (" + table.getName() + ")");
                        for (DbEntity column : ((Table) table).getAllEntities()) {
                            System.out.println(column.getName() + " " + ((Column) column).getDatatype());
                            assertTrue(column.getStatus() == Status.SELECTED_SAVED || column.getStatus() == Status.NOT_SELECTED);
                        }
                        break;
                    }
                }

                break;
            }
        }

        // Test loading new schema and table (first empty result)
        boolean hasLoadedTable = false;
        for (String schemaName : tapSchema.getSchemasNames()) {
            Schema schema = tapSchema.getSchema(schemaName);
            if (schema == null) {
                ArrayList<String> selectedSchemas = new ArrayList<String>();
                selectedSchemas.add(schemaName);
                tapSchema.loadEntities(connection, selectedSchemas);
                schema = tapSchema.getSchema(schemaName);
                assertEquals(schema.getStatus(), Status.SELECTED_TO_SAVE);
                
                for (String tableName : schema.getAddables()) {
                    Table table = schema.getTable(tableName);
                    if (table == null) {
                        ArrayList<String> selectedTables = new ArrayList<String>();
                        selectedTables.add(tableName);
                        schema.loadEntities(connection, selectedTables);
                        table = schema.getTable(tableName);
                        assertEquals(table.getStatus(), Status.SELECTED_TO_SAVE);

                        for (DbEntity column : table.getAllEntities()) {
                            assertEquals(column.getStatus(), Status.SELECTED_TO_SAVE);
                        }

                        hasLoadedTable = true;
                        break;
                    }
                }

                break;
            }
        }
        assertTrue(hasLoadedTable);
    }

    private static void printArray(ArrayList<String> list) {
        for (int i = 0; i < list.size(); i++) {
            System.out.print(list.get(i));
            if (i < list.size() - 1) {
                System.out.print(", ");
            }
        }
        System.out.print("\n\n");
    }
}
