package it.inaf.oats.ia2.tap.utils;

import java.io.Serializable;
import java.util.ArrayList;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.request.resource.UrlResourceReference;

/**
 *
 * @author pietro
 * @version
 */
public abstract class BasePage extends WebPage {

    private static final JavaScriptResourceReference BOOTSTRAP_REF = new JavaScriptResourceReference(BasePage.class, "js/bootstrap.min.js");
    private static final JavaScriptResourceReference MAIN_REF = new JavaScriptResourceReference(BasePage.class, "js/main.js");
    
    private final Label titleLabel;
    private final String title;
    private final WebMarkupContainer alertPanel;

    private final ArrayList<Alert> alerts;

    class Alert implements Serializable {

        final String message;
        final AlertType type;

        Alert(String message, AlertType type) {
            this.message = message;
            this.type = type;
        }
    }

    public BasePage(String title) {
        super();

        this.title = title;
        titleLabel = new Label("titleLabel", new Model<String>() {
            @Override
            public String getObject() {
                return BasePage.this.title;
            }
        });
        add(titleLabel);

        alertPanel = new WebMarkupContainer("alertPanel");
        alertPanel.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(alerts.size() > 0);
            }
        });
        alertPanel.setOutputMarkupPlaceholderTag(true);
        add(alertPanel);

        alerts = new ArrayList<Alert>();
        alertPanel.add(new ListView<Alert>("alert", alerts) {
            @Override
            protected void populateItem(final ListItem<Alert> item) {
                Alert alert = item.getModelObject();

                item.add(new Label("message", alert.message));

                String css = null;
                switch (alert.type) {
                    case INFO:
                        css = "success";
                        break;
                    case ERROR:
                        css = "danger";
                        break;
                    case WARNING:
                        css = "warning";
                        break;
                }
                item.add(new AttributeAppender("class", Model.of("alert-" + css), " "));

                item.add(new AjaxFallbackLink("close") {

                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        alerts.remove(item.getIndex());
                        target.add(alertPanel);
                    }
                });
            }
        });
    }

    @Override
    public void renderHead(IHeaderResponse response) {
        super.renderHead(response);

        // Always add jQuery
        response.render(JavaScriptHeaderItem.forReference(getApplication()
                .getJavaScriptLibrarySettings().getJQueryReference()));
        
        response.render(JavaScriptHeaderItem.forReference(BOOTSTRAP_REF));
        response.render(JavaScriptHeaderItem.forReference(MAIN_REF));
    }

    public void addInfo(String message) {
        alerts.add(new Alert(message, AlertType.INFO));
    }

    public void addInfo(String message, AjaxRequestTarget target) {
        addInfo(message);
        target.add(alertPanel);
    }

    public void addError(String message) {
        alerts.add(new Alert(message, AlertType.ERROR));
    }

    public void addError(String message, AjaxRequestTarget target) {
        addError(message);
        target.add(alertPanel);
    }
}
