package it.inaf.oats.ia2.tap.utils.model;

import java.io.Serializable;

/**
 * Object that keeps track of changes in the value that it wraps.
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 * @param <T> Class of the value contained in the object.
 */
public class EditableValue<T extends Serializable> implements Serializable {

    private final T originalValue;
    private T newValue;

    public EditableValue(T value) {
        originalValue = value;
        newValue = value;
    }

    public void setValue(T value) {
        newValue = value;
    }

    public T getValue() {
        return newValue;
    }

    public boolean isChanged() {
        return !originalValue.equals(newValue);
    }
}
