package it.inaf.oats.ia2.tap.utils.model;

import it.inaf.oats.ia2.tap.utils.model.Schema;
import it.inaf.oats.ia2.tap.utils.datalayer.ResultSetReader;
import it.inaf.oats.ia2.tap.utils.datalayer.TapSchemaDL;
import it.inaf.oats.ia2.tap.utils.model.Status;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author pietro
 */
public class TapSchema implements Serializable {

    private final String name;
    private boolean toCreate;
    //private final ArrayList<String> allSchemas;

    // current schemas (saved, added or to remove)
    private final Map<String, Schema> schemasMap;

    // For EXISTING TAP Schemas
    public TapSchema(Connection connection, String tapSchemaName) throws SQLException {
        this(connection, tapSchemaName, false, null);
        toCreate = false;
    }

    // For NEW TAP Schemas
    public TapSchema(Connection connection, String tapSchemaName, ArrayList<String> selectedSchemas) throws SQLException {
        this(connection, tapSchemaName, true, selectedSchemas);
    }

    private TapSchema(final Connection connection, String tapSchemaName, boolean toCreate, ArrayList<String> selectedSchemas) throws SQLException {
        this.name = tapSchemaName;
        this.toCreate = toCreate;

        schemasMap = new HashMap<String, Schema>();
        for (String schemaName : TapSchemaDL.getAllSchemasNames(connection)) {
            schemasMap.put(schemaName, null);
        }

        if (toCreate) {
            loadSelectedSchemas(connection, selectedSchemas);
        } else {
            TapSchemaDL.loadSavedSchemas(connection, (this));
        }
    }

    public final void loadSelectedSchemas(Connection connection, ArrayList<String> selectedSchemas) throws SQLException {
        TapSchemaDL.loadSelectedSchemas(connection, this, selectedSchemas);
    }

    public final Schema loadSchema(Connection connection, String schemaName) throws SQLException {
        ArrayList<String> selectedSchemas = new ArrayList<String>();
        selectedSchemas.add(schemaName);
        loadSelectedSchemas(connection, selectedSchemas);
        return getSchema(schemaName);
    }
    
    public boolean toCreate() {
        return toCreate;
    }

    public Schema getSchema(String name) {
        return schemasMap.get(name);
    }

    public void addSchema(String name, Schema schema) {
        schemasMap.put(name, schema);
    }

    public Collection<String> getSchemasNames() {
        return schemasMap.keySet();
    }
    
    public Collection<Schema> getSchemas() {
        return schemasMap.values();
    }

    public String getName() {
        return name;
    }
    
    public ArrayList<String> getAddableSchemas() {
        ArrayList<String> addableSchemas = new ArrayList<String>();
        
        for (String schemaName : schemasMap.keySet()) {
            if(schemasMap.get(schemaName) == null && !schemaName.equals(this.name)) {
                addableSchemas.add(schemaName);
            }
        }
        
        return addableSchemas;
    }
    
    public boolean removeSchema(String schemaName) {
        Schema schema = getSchema(schemaName);
        if(schema.getStatus() == Status.SELECTED_TO_SAVE) {
            schemasMap.put(schemaName, null);
        }
        if(schema.getStatus() == Status.SELECTED_SAVED) {
            schema.setStatus(Status.TO_REMOVE);
            return false;
        }
        return true;
    }
}
