package it.inaf.oats.ia2.tap.utils.panel;

import it.inaf.oats.ia2.tap.utils.DbEntityTab;
import it.inaf.oats.ia2.tap.utils.model.ITabsManipulator;
import it.inaf.oats.ia2.tap.utils.model.Status;
import java.awt.Container;
import java.util.List;
import org.apache.wicket.Component;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.model.Model;

/**
 * Tabbed panel with removable tabs
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class DynamicTabPanel extends AjaxTabbedPanel {

    private final List<DbEntityTab> tabs;
    private final ITabsManipulator tabsManipulator;

    public DynamicTabPanel(String id, List<DbEntityTab> tabs, final ITabsManipulator tabsManipulator) {
        super(id, tabs);
        this.tabs = tabs;
        this.tabsManipulator = tabsManipulator;

        WebMarkupContainer tabsContainer = (WebMarkupContainer) get("tabs-container");
        tabsContainer.add(new AjaxFallbackLink("addTab") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                tabsManipulator.addTab(target);
            }
        });
    }

    @Override
    protected String getSelectedTabCssClass() {
        // Bootstrap class for selected tab
        return "active";
    }

    @Override
    protected WebMarkupContainer newLink(String linkId, final int index) {
        WebMarkupContainer link = super.newLink(linkId, index);
        link.add(new AttributeAppender("class", new Model<String>() {
            @Override
            public String getObject() {
                return tabs.get(index).getStatus() == Status.TO_REMOVE ? "to-remove" : "";
            }
        }, " "));
        link.add(new AjaxLink("removeTab") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                tabsManipulator.removeTab(target, tabs.get(index).getTitle().getObject());
            }
        });
        return link;
    }
}
