package it.inaf.oats.ia2.tap.utils.panel;

import it.inaf.oats.ia2.tap.utils.DbEntityTab;
import it.inaf.oats.ia2.tap.utils.SchemaPage;
import it.inaf.oats.ia2.tap.utils.SelectSchemataPage;
import it.inaf.oats.ia2.tap.utils.panel.SearchUCDDialog;
import it.inaf.oats.ia2.tap.utils.model.ColumnField;
import it.inaf.oats.ia2.tap.utils.model.NullableView;
import it.inaf.oats.ia2.tap.utils.model.Column;
import it.inaf.oats.ia2.tap.utils.model.ITabsManipulator;
import it.inaf.oats.ia2.tap.utils.model.Schema;
import it.inaf.oats.ia2.tap.utils.model.Status;
import it.inaf.oats.ia2.tap.utils.model.Table;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.logging.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class SchemaPanel extends Panel implements ITabsManipulator {

    static final Logger log = Logger.getLogger(SelectSchemataPage.class.getName());

    class ColumnSelector extends AjaxFallbackLink {

        private final int index;

        public ColumnSelector(String id, ListItem<Column> item) {
            super(id);
            this.index = item.getIndex();
            this.add(new Label("columnName", item.getModelObject().getColumnName()));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            SchemaPanel.this.selectColumn(index);
            target.add(SchemaPanel.this);
        }
    }

    private final SchemaPage schemaPage;
    private final Schema schema;

    Table selectedTable = null;
    final ArrayList<String> tablesNames;
    //final Model<String> selectedTableModel;
    //final DropDownChoice tableDropDownChoice;

    int selectedColumnIndex = 0;
    //final ArrayList<Column> columns;
    Column selectedColumn = null;

    //final NullableView<Column> selectedColumnView;
    //final SearchUCDDialog searchUCDDialog;
    TextField currentUCDTextField;
    final ArrayList<DbEntityTab> tablesTabs;
    final DynamicTabPanel tablesTabbedPanel;

    public SchemaPanel(final SchemaPage schemaPage, String id, final Schema schema) {
        super(id);
        this.schemaPage = schemaPage;
        this.schema = schema;
        setOutputMarkupPlaceholderTag(true);

        WebMarkupContainer toRemoveInfo = new WebMarkupContainer("toRemoveInfo");
        toRemoveInfo.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(schema.getStatus() == Status.TO_REMOVE);
            }
        });
        add(toRemoveInfo);
        toRemoveInfo.add(new AjaxLink("undoRemove") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                schema.setStatus(Status.SELECTED_SAVED);
                target.add(schemaPage);
            }
        });

        WebMarkupContainer schemaWrapper = new WebMarkupContainer("schemaWrapper");
        schemaWrapper.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(schema.getStatus() != Status.TO_REMOVE);
            }
        });
        add(schemaWrapper);

        TextField<String> schemaUtypeTextField
                = new TextField<String>("schemaUtype", new Model<String>() {

                    @Override
                    public String getObject() {
                        return SchemaPanel.this.schema.getUtype();
                    }

                    @Override
                    public void setObject(String object) {
                        SchemaPanel.this.schema.setUtype(object);
                    }
                });
        schemaWrapper.add(schemaUtypeTextField);

        TextField<String> schemaDescriptionTextField
                = new TextField<String>("schemaDescription", new Model<String>() {

                    @Override
                    public String getObject() {
                        return SchemaPanel.this.schema.getDescription();
                    }

                    @Override
                    public void setObject(String object) {
                        SchemaPanel.this.schema.setDescription(object);
                    }
                });
        schemaWrapper.add(schemaDescriptionTextField);

        tablesNames = new ArrayList<String>();
        for (String tableName : schema.getTablesNames()) {
            tablesNames.add(tableName);
        }

        tablesTabs = new ArrayList<DbEntityTab>();
        for (Table table : schema.getTables()) {
            if (table != null) {
                tablesTabs.add(new DbEntityTab(schemaPage, "panel", table));
            }
        }

        tablesTabbedPanel = new DynamicTabPanel("tablesTabbedPanel", tablesTabs, this);
        tablesTabbedPanel.setOutputMarkupPlaceholderTag(true);
        schemaWrapper.add(tablesTabbedPanel);

        /*
         selectedTableModel = new Model<String>() {

         private String selectedTableName = null;

         @Override
         public String getObject() {
         return selectedTableName;
         }

         @Override
         public void setObject(String object) {
         selectedTableName = object;
         if (selectedTableName != null) {
         selectedTable = SchemaPanel.this.schema.getTable(selectedTableName);
         columns.clear();
         for (Column column : selectedTable.getColumnsMap().values()) {
         columns.add(column);
         }

         selectColumn(0);
         }
         }
         };

         tableDropDownChoice = new DropDownChoice<String>(
         "tableDropDownChoice", selectedTableModel, tablesNames);
         tableDropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
         @Override
         protected void onUpdate(AjaxRequestTarget target) {
         target.add(SchemaPanel.this);
         }
         });
         schemaWrapper.add(tableDropDownChoice);

         WebMarkupContainer tableInfo = new WebMarkupContainer("tableInfo");
         tableInfo.add(new Behavior() {
         @Override
         public void onConfigure(Component component) {
         component.setVisible(selectedTableModel.getObject() != null);
         }
         });
         schemaWrapper.add(tableInfo);

         tableInfo.add(new TextField<String>("tableUtype", new Model<String>() {

         @Override
         public String getObject() {
         if (selectedTable != null) {
         return selectedTable.getUtype();
         }
         return null;
         }

         @Override
         public void setObject(String object) {
         if (selectedTable != null) {
         selectedTable.setUtype(object);
         }
         }
         }));
         tableInfo.add(new TextField<String>("tableDescription", new Model<String>() {

         @Override
         public String getObject() {
         if (selectedTable != null) {
         return selectedTable.getDescription();
         }
         return null;
         }

         @Override
         public void setObject(String object) {
         if (selectedTable != null) {
         selectedTable.setDescription(object);
         }
         }
         }));
         tableInfo.add(new AjaxCheckBox("tableSelected",
         new Model<Boolean>() {
         @Override
         public Boolean getObject() {
         if (selectedTable != null) {
         return selectedTable.isChecked();
         }
         return false;
         }

         @Override
         public void setObject(Boolean checked) {
         if (selectedTable != null) {
         selectedTable.setChecked(checked);
         schema.verifyCheckedState();
         }
         }
         }) {

         @Override
         protected void onUpdate(AjaxRequestTarget target) {
         target.add(SchemaPanel.this);
         }
         });

         WebMarkupContainer columnsContainer = new WebMarkupContainer("columnsContainer");
         columnsContainer.add(new Behavior() {
         @Override
         public void onConfigure(Component component) {
         component.setVisible(selectedColumn != null);
         }
         });
         schemaWrapper.add(columnsContainer);

         columns = new ArrayList<Column>();
         columnsContainer.add(new ListView<Column>("columnsList", columns) {
         @Override
         protected void populateItem(ListItem<Column> item) {
         item.add(new ColumnSelector("columnLink", item));

         final Column column = item.getModelObject();
         item.add(new AjaxCheckBox("columnChecked",
         new Model<Boolean>() {
         @Override
         public Boolean getObject() {
         return column.isChecked();
         }

         @Override
         public void setObject(Boolean checked) {
         column.setChecked(checked);
         }
         }) {
         @Override
         protected void onUpdate(AjaxRequestTarget target) {
         selectedTable.verifyCheckedState();
         schema.verifyCheckedState();
         target.add(SchemaPanel.this);
         }
         });

         if (selectedColumnIndex == item.getIndex()) {
         item.add(new AttributeModifier("class", "active"));
         }
         }
         });

         selectedColumnView = new NullableView<Column>("selectedColumn") {

         @Override
         protected void populateItem(ListItem<Column> li) {
         li.add(new ColumnPropertyTextField<String>("utype", ColumnField.UTYPE));

         ColumnPropertyTextField ucd = new ColumnPropertyTextField<String>("ucd", ColumnField.UCD);
         ucd.add(new AjaxFormComponentUpdatingBehavior("onclick") {
         @Override
         protected void onUpdate(AjaxRequestTarget target) {
         target.add(SchemaPanel.this);
         SchemaPanel.this.searchUCDDialog.setColumn(selectedColumn);
         target.appendJavaScript("TSM.openModal('ucd-dialog');");
         }
         });
         ucd.setOutputMarkupPlaceholderTag(true);
         currentUCDTextField = ucd;
         li.add(ucd);

         li.add(new ColumnPropertyTextField<String>("unit", ColumnField.UNIT));
         li.add(new ColumnPropertyTextField<String>("description", ColumnField.DESCRIPTION));
         li.add(new ColumnPropertyTextField<String>("datatype", ColumnField.DATATYPE));
         }
         };

         columnsContainer.add(selectedColumnView);

         searchUCDDialog = new SearchUCDDialog(schemaPage, this, "searchUCDDialog");
         schemaWrapper.add(searchUCDDialog);*/
    }

    private void selectColumn(int index) {
//        if (columns.size() > 0) {
//            selectedColumnIndex = index;
//            selectedColumn = columns.get(index);
//            selectedColumnView.setContent(selectedColumn);
//        } else {
//            selectedColumnView.clear();
//            selectedColumn = null;
//
//        }
    }

//    class ColumnPropertyTextField<T extends Serializable> extends TextField<T> {
//
//        protected ColumnPropertyTextField(final String id, final ColumnField key) {
//            super(id, selectedColumn.getPropertyModel(key));
//
//            add(new AttributeAppender("class", new Model<String>() {
//                @Override
//                public String getObject() {
//                    return selectedColumn.isChanged(key) ? "changed" : "";
//                }
//            }, " "));
//
//            // TextField class is updated using JavaScript to avoid it will be 
//            // rendered again (and lose caret position inside textfield)
//            add(new OnChangeAjaxBehavior() {
//                @Override
//                protected void onUpdate(final AjaxRequestTarget target) {
//                    target.appendJavaScript("TSM.setChanged('" + id + "'," + selectedColumn.isChanged(key) + ");");
//                }
//            });
//        }
//    }
//
//    public TextField getUCDTextField() {
//        return currentUCDTextField;
//    }

    @Override
    public void addTab(AjaxRequestTarget target) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void removeTab(AjaxRequestTarget target, String tabName) {
        boolean removed = schema.removeTable(tabName);
        if(removed) {
            // TODO
        }
        target.add(this);
    }
}
