package it.inaf.oats.ia2.tap.utils.panel;

import it.inaf.oats.ia2.tap.utils.SchemaPage;
import it.inaf.oats.ia2.tap.utils.model.Column;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class SearchUCDDialog extends Panel {

    //static final Logger log = Logger.getLogger(SearchUCDDialog.class.getName());
    static final String UCD_REST_SERVICE = "http://localhost:8080/UCDRest/";
    static final String UCD_NOT_FOUND = "**** Could not find UCD ****";

    class UCDInfo implements Serializable {

        String score;
        String flag;
        String word;
        String description;

        UCDInfo(String score, String flag, String word) {
            this.score = score;
            this.flag = flag;
            this.word = word;
            this.description = "";
        }
    }

    private Column column;

    final WebMarkupContainer modalContent;
    String selectedUCD, assignedUCD, description;
    final ArrayList<UCDInfo> suggestedUCD;
    boolean UCDnotFound;

    public SearchUCDDialog(final SchemaPage schemaPage, final TablePanel tablePanel, String id) {
        super(id);

        suggestedUCD = new ArrayList<UCDInfo>();
        reset();

        modalContent = new WebMarkupContainer("modalContent");
        modalContent.setOutputMarkupPlaceholderTag(true);
        add(modalContent);

        modalContent.add(new TextField<String>("ucdDialogDescription", new Model<String>() {
            @Override
            public String getObject() {
                return description;
            }

            @Override
            public void setObject(String value) {
                description = value;
            }
        }));

        modalContent.add(new AjaxSubmitLink("searchUCD") {
            @Override
            protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                reset();

                String error = null;
                try {
                    String searchText = URLEncoder.encode(description, "UTF-8").replace("+", "%20");

                    // Search assign value
                    URL urlAssign = new URL(UCD_REST_SERVICE + "assign/" + searchText);

                    HttpURLConnection connectionAssign = (HttpURLConnection) urlAssign.openConnection();

                    int responseCodeAssign = connectionAssign.getResponseCode();
                    if (responseCodeAssign == 200) {
                        BufferedReader br;
                        String line, response = "";

                        br = new BufferedReader(new InputStreamReader(connectionAssign.getInputStream()));
                        while ((line = br.readLine()) != null) {
                            response += line;
                        }
                        if (response.equals(UCD_NOT_FOUND)) {
                            UCDnotFound = true;
                        } else {
                            assignedUCD = response;
                            selectedUCD = assignedUCD;

                            // Search suggested values
                            URL urlSuggest = new URL(UCD_REST_SERVICE + "suggest/" + searchText);

                            HttpURLConnection connectionSuggest = (HttpURLConnection) urlSuggest.openConnection();

                            int responseCodeSuggest = connectionSuggest.getResponseCode();
                            if (responseCodeSuggest == 200) {
                                br = new BufferedReader(new InputStreamReader(connectionSuggest.getInputStream()));
                                while ((line = br.readLine()) != null) {
                                    String[] split1 = line.split("\\|");
                                    if (split1.length == 2) {
                                        String score = split1[0].trim();
                                        String[] split2 = split1[1].trim().split(" ");
                                        if (split2.length == 2) {
                                            String flag = split2[0];
                                            String word = split2[1];
                                            suggestedUCD.add(new UCDInfo(score, flag, word));
                                        }
                                    }
                                }
                            } else if (responseCodeSuggest == 204) {
                                UCDnotFound = true;
                            } else {
                                error = "Response code: " + responseCodeSuggest;
                            }
                        }
                    } else if (responseCodeAssign == 204) {
                        UCDnotFound = true;
                    } else {
                        error = "Response code: " + responseCodeAssign;
                    }
                } catch (MalformedURLException e) {
                    error = e.getMessage();
                } catch (IOException e) {
                    error = e.getMessage();
                }

                if (error != null) {
                    schemaPage.addError("Error while retrieving data. " + error);
                }
                target.add(modalContent);
            }
        });

        // UCD Not found alert
        WebMarkupContainer UCDnotFoundAlert = new WebMarkupContainer("UCDnotFound");
        UCDnotFoundAlert.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(UCDnotFound);
            }
        });
        modalContent.add(UCDnotFoundAlert);

        // Selected UCD
        modalContent.add(new Label("selectedUCD", new Model<String>() {
            @Override
            public String getObject() {
                return selectedUCD;
            }
        }));

        // Assigned UCD Button
        AjaxSubmitLink assignedUCDBtn = new AjaxSubmitLink("assignedUCDBtn") {
            @Override
            protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                selectedUCD = assignedUCD;
                target.add(modalContent);
            }
        };
        modalContent.add(assignedUCDBtn);
        assignedUCDBtn.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(!UCDnotFound && !assignedUCD.equals(""));
            }
        });
        assignedUCDBtn.add(new Label("assignedUCD", new Model<String>() {
            @Override
            public String getObject() {
                return assignedUCD;
            }
        }));

        // Suggested UCD list
        modalContent.add(new ListView<UCDInfo>("ucdSuggestResult", suggestedUCD) {
            @Override
            protected void populateItem(final ListItem<UCDInfo> item) {
                final UCDInfo values = item.getModelObject();

                item.add(new Label("score", values.score));

                Label flag = new Label("flag", values.flag);
                flag.add(new AttributeAppender("class", new Model("flag" + values.flag), " "));
                item.add(flag);

                AjaxSubmitLink suggestedUCDBtn = new AjaxSubmitLink("suggestedUCDBtn") {
                    @Override
                    protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                        selectedUCD = values.word;
                        target.add(modalContent);
                    }
                };
                item.add(suggestedUCDBtn);
                suggestedUCDBtn.add(new Label("word", values.word));

                item.add(new Label("definition", new Model<String>() {
                    @Override
                    public String getObject() {
                        return values.description;
                    }
                }));

                item.add(new AjaxSubmitLink("findDefinition") {
                    @Override
                    protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                        String error = null;
                        try {
                            String searchText = URLEncoder.encode(values.word, "UTF-8").replace("+", "%20");
                            URL url = new URL(UCD_REST_SERVICE + "explain/" + searchText);

                            HttpURLConnection connection = (HttpURLConnection) url.openConnection();

                            int responseCode = connection.getResponseCode();
                            if (responseCode == 200) {
                                BufferedReader br = new BufferedReader(new InputStreamReader(connection.getInputStream()));
                                String line, response = "";
                                while ((line = br.readLine()) != null) {
                                    response += line;
                                }

                                values.description = response;
                                target.add(modalContent);
                            } else {
                                error = "Response code: " + responseCode;
                            }
                        } catch (MalformedURLException e) {
                            error = e.getMessage();
                        } catch (IOException e) {
                            error = e.getMessage();
                        }

                        if (error != null) {
                            schemaPage.addError("Error while retrieving data. " + error);
                        }
                    }
                });
            }
        });

        modalContent.add(new AjaxSubmitLink("ucdDialogSaveBtn") {
            @Override
            protected void onSubmit(final AjaxRequestTarget target, Form<?> form) {
                column.setUcd(selectedUCD);
                target.add(tablePanel.getUCDTextField());
                target.appendJavaScript("TSM.closeModal('ucd-dialog');");
            }
        });
    }

    public void setColumn(Column column) {
        reset();

        this.column = column;
        if (column.getUcd() != null) {
            selectedUCD = column.getUcd();
        }
        description = (column.getDescription() != null) ? column.getDescription() : "";
    }

    private void reset() {
        selectedUCD = "";
        assignedUCD = "";
        suggestedUCD.clear();
        UCDnotFound = false;
    }
}
