package it.inaf.oats.ia2.tap.utils.panel;

import it.inaf.oats.ia2.tap.utils.SchemaPage;
import it.inaf.oats.ia2.tap.utils.SelectSchemataPage;
import it.inaf.oats.ia2.tap.utils.panel.SearchUCDDialog;
import it.inaf.oats.ia2.tap.utils.model.ColumnField;
import it.inaf.oats.ia2.tap.utils.model.NullableView;
import it.inaf.oats.ia2.tap.utils.model.Column;
import it.inaf.oats.ia2.tap.utils.model.Schema;
import it.inaf.oats.ia2.tap.utils.model.Status;
import it.inaf.oats.ia2.tap.utils.model.Table;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.logging.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class TablePanel extends Panel {

    static final Logger log = Logger.getLogger(TablePanel.class.getName());

    class ColumnSelector extends AjaxFallbackLink {

        private final Column column;

        public ColumnSelector(String id, ListItem<Column> item) {
            super(id);
            this.column = item.getModelObject();
            this.add(new Label("columnName", column.getColumnName()));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            TablePanel.this.selectColumn(column);
            target.add(TablePanel.this);
        }
    }

    private final SchemaPage schemaPage;
    private final Table table;

    Column selectedColumn = null;
    final NullableView<Column> selectedColumnView;

    final SearchUCDDialog searchUCDDialog;
    TextField currentUCDTextField;

    public TablePanel(final SchemaPage schemaPage, String id, final Table table) {
        super(id);
        setOutputMarkupPlaceholderTag(true);
        this.schemaPage = schemaPage;
        this.table = table;

        WebMarkupContainer toRemoveInfo = new WebMarkupContainer("toRemoveInfo");
        toRemoveInfo.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(table.getStatus() == Status.TO_REMOVE);
            }
        });
        add(toRemoveInfo);
        toRemoveInfo.add(new AjaxLink("undoRemove") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                table.setStatus(Status.SELECTED_SAVED);
                target.add(schemaPage);
            }
        });

        WebMarkupContainer tableWrapper = new WebMarkupContainer("tableWrapper");
        tableWrapper.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(table.getStatus() != Status.TO_REMOVE);
            }
        });
        add(tableWrapper);
//
//        tablesNames = new ArrayList<String>();
//        for (String tableName : schema.getTablesNames()) {
//            tablesNames.add(tableName);
//        }

//        selectedTableModel = new Model<String>() {
//
//            private String selectedTableName = null;
//
//            @Override
//            public String getObject() {
//                return selectedTableName;
//            }
//
//            @Override
//            public void setObject(String object) {
//                selectedTableName = object;
//                if (selectedTableName != null) {
//                    selectedTable = SchemaPanel.this.schema.getTable(selectedTableName);
//                    columns.clear();
//                    for (Column column : selectedTable.getColumnsMap().values()) {
//                        columns.add(column);
//                    }
//
//                    selectColumn(0);
//                }
//            }
//        };
//        tableDropDownChoice = new DropDownChoice<String>(
//                "tableDropDownChoice", selectedTableModel, tablesNames);
//        tableDropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
//            @Override
//            protected void onUpdate(AjaxRequestTarget target) {
//                target.add(SchemaPanel.this);
//            }
//        });
//        schemaWrapper.add(tableDropDownChoice);
        tableWrapper.add(new TextField<String>("tableUtype", new Model<String>() {

            @Override
            public String getObject() {
                return table.getUtype();
            }

            @Override
            public void setObject(String object) {
                table.setUtype(object);
            }
        }));
        tableWrapper.add(new TextField<String>("tableDescription", new Model<String>() {

            @Override
            public String getObject() {
                return table.getDescription();
            }

            @Override
            public void setObject(String object) {
                table.setDescription(object);
            }
        }));
//        tableWrapper.add(new AjaxCheckBox("tableSelected",
//                new Model<Boolean>() {
//                    @Override
//                    public Boolean getObject() {
//                        if (selectedTable != null) {
//                            return selectedTable.isChecked();
//                        }
//                        return false;
//                    }
//
//                    @Override
//                    public void setObject(Boolean checked) {
//                        if (selectedTable != null) {
//                            selectedTable.setChecked(checked);
//                            schema.verifyCheckedState();
//                        }
//                    }
//                }) {
//
//                    @Override
//                    protected void onUpdate(AjaxRequestTarget target) {
//                        target.add(SchemaPanel.this);
//                    }
//                });

        WebMarkupContainer columnsContainer = new WebMarkupContainer("columnsContainer");
        columnsContainer.add(new Behavior() {
            @Override
            public void onConfigure(Component component) {
                component.setVisible(!table.getColumns().isEmpty());
            }
        });
        tableWrapper.add(columnsContainer);

        // Columns ListView
        columnsContainer.add(new ListView<Column>("columnsList", new Model<ArrayList<Column>>() {

            @Override
            public ArrayList<Column> getObject() {
                return new ArrayList(table.getColumns());
            }
        }) {
            @Override
            protected void populateItem(ListItem<Column> item
            ) {
                item.add(new ColumnSelector("columnLink", item));

                final Column column = item.getModelObject();
                item.add(new AjaxCheckBox("columnChecked",
                        new Model<Boolean>() {
                            @Override
                            public Boolean getObject() {
                                return column.isChecked();
                            }

                            @Override
                            public void setObject(Boolean checked) {
                                column.setChecked(checked);
                            }
                        }) {
                            @Override
                            protected void onUpdate(AjaxRequestTarget target) {
                                table.verifyCheckedState();
                                //schema.verifyCheckedState();
                                //target.add(SchemaPanel.this);
                            }
                        });

                if (selectedColumn.getColumnName().equals(column.getColumnName())) {
                    item.add(new AttributeModifier("class", "active"));
                }
            }
        });

        selectedColumnView = new NullableView<Column>("selectedColumn") {

            @Override
            protected void populateItem(ListItem<Column> li) {
                li.add(new ColumnPropertyTextField<String>("utype", ColumnField.UTYPE));

                ColumnPropertyTextField ucd = new ColumnPropertyTextField<String>("ucd", ColumnField.UCD);
                ucd.add(new AjaxFormComponentUpdatingBehavior("onclick") {
                    @Override
                    protected void onUpdate(AjaxRequestTarget target) {
                        target.add(TablePanel.this);
                        TablePanel.this.searchUCDDialog.setColumn(selectedColumn);
                        target.appendJavaScript("TSM.openModal('ucd-dialog');");
                    }
                });
                ucd.setOutputMarkupPlaceholderTag(true);
                currentUCDTextField = ucd;
                li.add(ucd);

                li.add(new ColumnPropertyTextField<String>("unit", ColumnField.UNIT));
                li.add(new ColumnPropertyTextField<String>("description", ColumnField.DESCRIPTION));
                li.add(new ColumnPropertyTextField<String>("datatype", ColumnField.DATATYPE));
            }
        };
        columnsContainer.add(selectedColumnView);

        if (!table.getColumns().isEmpty()) {
            // select first column by default
            selectColumn(table.getColumns().iterator().next());
        }
        
        searchUCDDialog = new SearchUCDDialog(schemaPage, this, "searchUCDDialog");
        tableWrapper.add(searchUCDDialog);
    }

    private void selectColumn(Column column) {
        if (table.getColumns().isEmpty()) {
            selectedColumnView.clear();
            selectedColumn = null;
        } else {
            selectedColumn = column;
            selectedColumnView.setContent(selectedColumn);
        }
    }

    class ColumnPropertyTextField<T extends Serializable> extends TextField<T> {

        protected ColumnPropertyTextField(final String id, final ColumnField key) {
            super(id, selectedColumn.getPropertyModel(key));

            add(new AttributeAppender("class", new Model<String>() {
                @Override
                public String getObject() {
                    return selectedColumn.isChanged(key) ? "changed" : "";
                }
            }, " "));

            // TextField class is updated using JavaScript to avoid it will be 
            // rendered again (and lose caret position inside textfield)
            add(new OnChangeAjaxBehavior() {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    target.appendJavaScript("TSM.setChanged('" + id + "'," + selectedColumn.isChanged(key) + ");");
                }
            });
        }
    }

    public TextField getUCDTextField() {
        return currentUCDTextField;
    }
}
